package de.hft.stuttgart.citydoctor2.connect.edge;

/**
 * This class is part of the JNI interface which controls the access to the
 * check- and heal kernel in native C++ code.<br>
 * It's represent a base class for classes which encapsulates a C++ object.
 * Therefor two abstract methods {@link createCppObject} and
 * {@link disposeCppObject} are defined, to create and dispose a C++ object. All
 * non-abstract subclasses are requested to implement at least the two native
 * methods<br>
 * <br>
 * <code> 
 * native long create[MeaningfulName](long ptrToCppFeature);<br>
 * native void dispose[MeaningfulName](long ptrToCppPolygonCheck);
 * </code><br>
 * <br>
 * 
 * @author wewetzer
 * @version 1.0
 * 
 */
public abstract class CppReferenceHandlingBase {
	
	// This long holds the address of an C++ object
	long ptrToCppObject = 0L;

	/**
	 * This method frees the occupied memory of the created C++ object. The method
	 * should be called after the instance of an child class is nor longer needed,
	 * to avoid memory leaks.
	 * 
	 * @see {@link createCppObject}
	 */
	public abstract void disposeCppObject();

	/**
	 * Checks if the pointer to the corresponding C++ object is 0. In this case an
	 * exception is thrown. This method should be called at first in every method,
	 * that uses native methods!
	 * 
	 * @throws IllegalArgumentException if the pointer to the corresponding C++
	 *                                  object i 0
	 */
	public void checkNullPointer() {
		if (0L == ptrToCppObject) {
			throw new IllegalArgumentException("C++ object pointer is null");
		}
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		CppReferenceHandlingBase other = (CppReferenceHandlingBase) obj;
		return ptrToCppObject == other.ptrToCppObject;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#finalize()
	 */
	@Override
	protected void finalize() throws Throwable {
		if (0L != ptrToCppObject) {
			disposeCppObject();
		}
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (int) (ptrToCppObject ^ (ptrToCppObject >>> 32));
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return ("ptrToCppObject\n\t" + ptrToCppObject + "\n");
	}

}
